function [Error_Percentage_Mean,Error_Percentage_Std]=ECV2(DataSet,numings_per_subject,approach,rowdiv,coldiv)

%%%%    Written by Liang Chen (lchen AT ieee.org), Aug./2008
%Input:
%   DataSet 		-- the entire face set, in our experiment
%             		   we use 'ORL' , or 'Yale'
%   numings_per_subject 	-- number of images per subject for trainning
%                               (also as gallary), remainings for testing
%                               for TrainDir='../x/kTrain/', 
%                               (x='ORL' or 'Yale')
%                               numbers_per_subject=k
%   approach		-- the holistic algorithms used in Electoral College
%                       we can choose here
%                       PCA
%                       SRDA
%                       S-LDA
%                       S-LPP
%			fisherface
%%
%   rowdiv and coldiv       -- we partition the face image into 
%                               a total of rowdiv x coldiv windows
%                               if coldiv is not given, we set coldiv = rowdiv
%                               please make sure 
%                               rowdiv <= 0.5(image height) - 2
%                               rowdiv <= 0.5(image height) - 2 
%			                    (2 is the space in each side I leave for shifting).
%                               (see lines 46 and 47)
%
%Output:
%   Error_Percentage_Mean -- The Average Erorr Rate for all 50 diffierent splits
%   Error_Percentage_Std  -- The Average Erorr Rate for all 50 diffierent splits
%
%Special Case: when both rowdiv and coldiv are not given, original approach without perturbing is applied.
%
% Example:
%          [MeanValue,StdValue]=ECV2('ORL',2,'PCA',3)
%	   [MeanValue,StdValue]=ECV2('ORL',2,'PCA')
% Notice: since we run all 50 splits provided by UIUC 
% http://www.cs.uiuc.edu/homes/dengcai2/Data/data.html
% it may take lots of time. 
% If you wish, you could modify line 79  of this program
% so as to only run N (N<50) different splits.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Shifts allowed in row and colume directions
shiftr=2;			      %Line 46
shiftc=2;			      %Line 47

if ~exist('rowdiv','var')     %original "approach" without perturbing
    rowdiv=1;
    shiftr=0;
    shiftc=0;
end

if ~exist('coldiv','var')
    coldiv=rowdiv;
end

switch lower(DataSet)
    case {lower('ORL')}       %1. ORL:
        load('ORL/ORL_64x64.mat');
    case {lower('YALE')}       %2 Yale
        load('Yale/Yale_64x64.mat');
    otherwise
        error('DataSet does not exist!');
end;

%   TrainDir 		-- the train/test split set

TrainDir=[DataSet,'/',int2str(numings_per_subject),'Train']

%%% Size of images: irow*icol
irow=64;
icol=irow;  %irow x icol is image size

Totalshift=(2*shiftr+1)*(2*shiftc+1);

%%% Total "random" splits in the kTrain set
totaltests=50;                        %Line 79

switch lower(DataSet)
    case {lower('ORL')}       %1. ORL:
        num_subjects=40;
        numimgs_gallary=numings_per_subject*num_subjects;
        numTestImages_per_subject=10-numings_per_subject;
        numimgs_test=num_subjects*numTestImages_per_subject;
    case {lower('YALE')}       %2 Yale
        num_subjects=15;
        numimgs_gallary=numings_per_subject*num_subjects;
        numTestImages_per_subject=11-numings_per_subject;
        numimgs_test=num_subjects*numTestImages_per_subject;
    otherwise
        error('DataSet does not exist!');
end;

%%% number of all the images
totalimages=size(fea,1);

fea=reshape(fea',irow,icol,totalimages);  %recover the image : from vector to 2D array


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%I try to find all the correct answers here,
%so that, I can easily check the answers to find out the error rates
AnswerSheet=false(num_subjects,numimgs_test);
for i=1:num_subjects
    for j=1:numimgs_test
        if (j>(i-1)*numTestImages_per_subject) && (j<=i*numTestImages_per_subject)
            AnswerSheet(i,j)=true;
        end;
    end;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

allcorrectansers=zeros(totaltests,1);
allcorrectansersnotie=zeros(totaltests,1,'uint16');
allwronganserspercentage=zeros(totaltests,1);

warning off;

for idt=1:totaltests
    load([TrainDir,'/',int2str(idt),'.mat']);
    trainIdx=sort(trainIdx);
    testIdx=sort(testIdx);
    trains=fea(:,:,trainIdx);
    tests=fea(:,:,testIdx);

         
    totalvotes=zeros(num_subjects,numimgs_test);
        
    for r=1:rowdiv
        for c=1:coldiv
                                      
                R0=shiftr+round((irow-2*shiftr)*(r-1)/rowdiv)+1; % We leave shiftr and shiftc in each side for shifting purpose
                C0=shiftc+round((icol-2*shiftc)*(c-1)/coldiv)+1;
                R1=shiftr+round((irow-2*shiftr)*r/rowdiv);
                C1=shiftc+round((icol-2*shiftc)*c/coldiv);
                 
                actualsize=(R1-R0+1)*(C1-C0+1);
                trainsample=(squeeze(reshape(trains(R0:R1,C0:C1,:),actualsize,1,[])))';
	
                trainsample=normr(trainsample);

%%%%%% in case all the pixels are "0", there will be "NoN" after normr, 
%%%%%% we shall have to set it 1/sqrt(actualsize)
                TF=isnan(trainsample);
                nanID=find(TF==1);
                if length(nanID)>0
                    trainsample(nanID)=1/sqrt(actualsize);
                end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



                %%%%%%%%%%%%%%%%%%%%%Choose One of the Following Approaches
                switch lower(approach)
                    case {lower('PCA')}       %1. PCA:
                        [nSmp,nFea]=size(trainsample);
                        ReducedDim=min(nSmp,nFea)-1;
                        [eigvector,eigvalue, elapse] = PCA(trainsample,ReducedDim);
                    case {lower('SRDA')}	  %2. SRDA
                        options = []; 		
                        options.gnd = gnd(trainIdx); 
                        options.ReguType = 'Ridge';
                        options.ReguAlpha = 0.01;
                        [eigvector] = SR_caller(options, trainsample);
                    case {lower('S-LDA')}      %3. S-LDA
                        gnd_Train = gnd(trainIdx);      
                        options = []; 
                        options.Regu = 1;
                        options.ReguAlpha = 0.01;
                        options.ReguType = 'Custom';
                        load(['tensorRs\TensorR',int2str(R1-R0+1),'by',int2str(C1-C0+1),'.mat']);
                        options.regularizerR = regularizerR;
                        [eigvector, eigvalue] = LDA(gnd_Train, options, trainsample);
                    case {lower('S-LPP')}       %4. S-LPP
                        gnd_Train = gnd(trainIdx);      
                        options = []; 
                        options.Metric = 'Cosine'; 
                        options.NeighborMode = 'Supervised'; 
                        options.WeightMode = 'Cosine'; 
                        options.gnd = gnd_Train; 
                        W = constructW(trainsample,options); 
                        options.Regu = 1;
                        options.ReguAlpha = 0.01;
                        options.ReguType = 'Custom';
                        load(['tensorRs\TensorR',int2str(R1-R0+1),'by',int2str(C1-C0+1),'.mat']);
                        options.regularizerR = regularizerR;
                        [eigvector, eigvalue] = LPP(W, options, trainsample);
                    case  {lower('FISHERFACE')}		%5. fisherface
                        options = [];
                        options.Fisherface = 1;
                        gnd_Train = gnd(trainIdx);
                        [eigvector, eigvalue] = LDA(gnd_Train, options, trainsample);
                    otherwise
                        error([approach,' does not exist!']);
                end;
                %%%%%%%%%%%%%%%%%%%%%%%%
                 
                VT=zeros(numimgs_gallary*Totalshift,actualsize);
                totalshift=0;
                for shiftrow=-shiftr:shiftr
                     for  shiftcol=-shiftc:shiftc; 
                         totalshift=totalshift+1;
                         indL=numimgs_gallary*(totalshift-1)+1;
                         indH=numimgs_gallary*totalshift;
                         VT(indL:indH,:)=(squeeze(reshape(trains(R0+shiftrow:R1+shiftrow,C0+shiftcol:C1+shiftcol,:),actualsize,1,[])))';
                     end;
                 end;
                  
                VT=normr(VT);
        %%%%%% in case all the pixels are "0", there will be "NoN" after normr, 
        %%%%%% we shall have to set it 1/sqrt(actualsize)
                TF=isnan(VT);
                nanID=find(TF==1);
                if length(nanID)>0
                    VT(nanID)=1/sqrt(actualsize);
                end;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


                GallaryWindow = VT*eigvector;
	         
                VT=[];

                VT=zeros(numimgs_test,actualsize);
                
                VT=(squeeze(reshape(tests(R0:R1,C0:C1,:),actualsize,1,[])))';
    	        VT=normr(VT);
            %%%%%% in case all the pixels are "0", there will be "NoN" after normr, 
            %%%%%% we shall have to set it 1/sqrt(actualsize)
                TF=isnan(VT);
                nanID=find(TF==1);
                if length(nanID)>0
                        VT(nanID)=1/sqrt(actualsize);
                end;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
                TestWindow = VT*eigvector;

                VT=[];
    
                tempsim = EuDist(GallaryWindow,TestWindow);
                
                simt=min(reshape(tempsim,[numimgs_gallary Totalshift numimgs_test]),[],2);
                tempsim=[];

                similaritygallary=squeeze(simt);
                simt=[];
                    
                similaritysubject_tmp= min(reshape(similaritygallary, [numings_per_subject num_subjects  numimgs_test]),[],1);
                similaritygallary=[];
                    
                similaritysubject=squeeze(similaritysubject_tmp);
                similaritysubject_tmp=[];

                M=min(similaritysubject,[],1);
                si=find(similaritysubject<=repmat(M+eps,num_subjects,1));
                                        
                totalvotes(si)=totalvotes(si)+1;    
                    
                similaritysubject=[];
   
                si=[];
                M=[];
             end;
         end;
         
         MT=max(totalvotes,[],1);
         IndexofMT=find(totalvotes==repmat(MT,num_subjects,1));
         totalvotes=0*totalvotes;
         totalvotes(IndexofMT)=1;
         counteachprobe=sum(totalvotes,1);
         totalvotes=totalvotes./repmat(counteachprobe,num_subjects,1);
 
         totalvotes=totalvotes.*AnswerSheet;
         
         nonzeroitems=nonzeros(totalvotes);
         rightanswer=sum(nonzeroitems);
         rightanswernotie=size(nonzeroitems,1)+sum(sign(nonzeroitems-1));
    
         allcorrectanswersnotie(idt)=rightanswernotie;
         allcorrectanswers(idt)=rightanswer;
         allwronganswerspercentage(idt)=(1-rightanswer/numimgs_test)*100;
end;

Error_Percentage_Mean=mean(allwronganswerspercentage);
Error_Percentage_Std=std(allwronganswerspercentage,1);